/* XNibbles - A simple X11 snake game
   Copyright (C) 1998 Sean MacIsaac and Ian Peters
  
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.
  
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY of FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
  
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  
   Specific questions about this program can be addressed to the authors,
   Sean Macisaac (sjm@acm.org), and Ian Peters (ipeters@acm.org). */

#include "xnibbles.h"

extern Display *display;
extern char objects[BWIDTH][BHEIGHT];
#if SOUND
extern Sound snds[SOUNDS];
extern int sound;
#endif
extern Window win;
extern boni *theBoni;
extern int warpx;
extern int warpy;
extern int level;
extern int channel;
extern int meltDelay;
extern int fps;
extern char* colors[];
extern XpmAttributes myXpmAttributes;
extern void error(int errorCode);

worm::worm(int aup, int adown, int aleft, int aright, char *aName, int x1, int y1,int which) {

  lives = 4;
  score = 0;
  dscore = 0;
  strcpy (worm_xpm[2],colors[6*which]);
  strcpy (worm_xpm[3],colors[6*which+1]);
  strcpy (worm_xpm[4],colors[6*which+2]);
  strcpy (worm_xpm[5],colors[6*which+3]);
  strcpy (worm_xpm[6],colors[6*which+4]);
  makePixmap (worm_xpm, &wormImage);
  meltCount = MELTSIZE;
  meltFlag = meltDelay;
  myBar = new statusBar(aName, x1, y1, which);
  head = NULL;
  spareHead = head;
  warpx = warpy = 1;
  length = 0;
  growing = 5;
  shrinking = 0;
  up = aup;
  down = adown;
  left = aleft;
  right = aright;
  flashCount = FLASHCOUNT;
  flashDelay = fps / 8;
}

void worm::newLevel(char x1, char y1, int dx1, int dy1) {
  bodyPart *end;

  if (head != NULL) {
    while (head->getNext() != NULL) {
      end = head;
      while (end->getNext()->getNext() != NULL)
        end = end->getNext();
      delete end->getNext();
      end->setNext(NULL);
    }
  }
  delete head;
  dx = dx1;
  dy = dy1;
  growing = 5;
  shrinking = 0;
  already = 0;
  length = 0;
  head = new bodyPart('w', x1, y1, NULL);
  spareHead = head;
}

worm::~worm() {
  bodyPart *end;

  while (head->getNext() != NULL) {
    end = head;
    while (end->getNext()->getNext() != NULL)
      end = end->getNext();
    delete end->getNext();
    end->setNext(NULL);
  }
  XFreePixmap(display, wormImage);
  delete head;
  delete myBar;
}

void worm::showStatusMaybe() {
  if (dscore > 0 && dscore < 100) {
    score += 1;
    dscore -= 1;
    myBar->update(lives, score, 3);
  } else
  if (dscore >= 100 && dscore < 1000) {
    score += 10;
    dscore -= 10;
    myBar->update(lives, score, 3);
  } else
  if (dscore >= 1000 && dscore < 10000) {
    score += 100;
    dscore -= 100;
    myBar->update(lives, score, 3);
  } else
  if (dscore >= 10000 && dscore < 100000) {
    score += 1000;
    dscore -= 1000;
    myBar->update(lives, score, 3);
  } else
  if (dscore >= 100000 && dscore < 1000000) {
    score += 10000;
    dscore -= 10000;
    myBar->update(lives, score, 3);
  } else
  if (dscore >= 1000000 && dscore < 10000000) {
    score += 100000;
    dscore -= 100000;
    myBar->update(lives, score, 3);
  } else
  if (dscore < 0 && dscore > (-100)) {
    score -= 1;
    dscore += 1;
    myBar->update(lives, score, 3);
  } else
  if (dscore <= (-100) && dscore > (-1000)) {
    score -= 10;
    dscore += 10;
    myBar->update(lives, score, 3);
  } else
  if (dscore <= (-1000) && dscore > (-10000)) {
    score -= 100;
    dscore += 100;
    myBar->update(lives, score, 3);
  } else
  if (dscore <= (-10000) && dscore > (-100000)) {
    score -= 1000;
    dscore += 1000;
    myBar->update(lives, score, 3);
  } else
  if (dscore <= (-100000) && dscore > (-1000000)) {
    score -= 10000;
    dscore += 10000;
    myBar->update(lives, score, 3);
  } else
  if (dscore <= (-1000000) && dscore > (-10000000)) {
    score -= 100000;
    dscore += 100000;
    myBar->update(lives, score, 3);
  }
}

void worm::showStatus(int which) {
  myBar->update(lives, score, which);
}

int worm::moveOrGrow() {
  if (shrinking && growing && (shrinking > growing)) {
    shrinking -= growing;
    growing = 0;
  } else
  if (shrinking && growing && (growing > shrinking)) {
    growing -= shrinking;
    shrinking = 0;
  } else
  if (shrinking && growing && (shrinking == growing)) {
    shrinking = growing = 0;
  }
  if (growing) {
    growing--;
    return addPartMove();
  } else {
    return move();
  }
}

int worm::move() {
  bodyPart *newHead;

  newHead = new bodyPart('w', head->getX() + dx, head->getY() + dy, head);
  head = newHead;
  if (!hit()) {
    newHead->draw(&wormImage);
    head->remove();
    if (shrinking) {
      head->remove();
      shrinking--;
    }
    already = 0;
    return 0;
  }
  head = newHead->getNext();
  delete newHead;
  return 1;
}

int worm::addPartMove() {
  bodyPart *newPart;

  newPart = new bodyPart('w', head->getX() + dx, head->getY() + dy, head);
  length++;
  head = newPart;
  spareHead = head;
  if (!hit()) {
    newPart->draw(&wormImage);
    already = 0;
    return 0;
  }
  head = newPart->getNext();
  spareHead = head;
  delete newPart;
  return 1;
}

void worm::drawAll() {
  bodyPart *temp;

  temp = head;
  if (temp->getNext() != NULL)
    while (temp != NULL) {
      temp->draw(&wormImage);
      temp = temp->getNext();
    }
}

void worm::eraseAll() {
  bodyPart *temp;

  temp = head;
  shrinking = 0;
  growing = 0;
  while (temp != NULL) {
    temp->erase();
    temp = temp->getNext();
  }
}

/*
int worm::melt() {
  bodyPart *temp;

  temp = spareHead;
  shrinking = 0;
  growing = 0;
  temp->erase();
  spareHead = temp->getNext();
  if (spareHead == NULL) return 0; else return 1;
}
*/

int worm::melt() {
  bodyPart *temp;
  int count = meltCount;

  if (!meltFlag) {
    temp = head;
    while (temp != NULL) {
      if (count == (MELTSIZE-1))
        temp->erase();
      count = (count + 1) % MELTSIZE;
      temp = temp->getNext();
    }
    meltFlag = meltDelay;
    meltCount--;
    if (!meltCount) {
      meltCount = MELTSIZE;
      meltFlag = meltDelay;
      return 0;
    } else return 1;
  } else {
    meltFlag--;
    return 1;
  }
}

int worm::changeDir(int key) {
  if (already) {
    return 0;
  }
  if ((key == up) && !dy) {
    dx = 0;
    dy = -1;
    already = 1;
  }
  if ((key == down) && !dy) {
    dx = 0;
    dy = 1;
    already = 1;
  }
  if ((key == left) && !dx) {
    dx = -1;
    dy = 0;
    already = 1;
  }
  if ((key == right) && !dx) {
    dx = 1;
    dy = 0;
    already = 1;
  }
  return 1;
}

int worm::hit() {
  char x, y;
  int xx, yy;

  x = head->getX();
  y = head->getY();
  if (objects[x][y] > 'a' && objects[x][y] < 'z') {
    lives--;
    dscore = (int)((float)score * 0.85) - score;
    return 1;
  }
  if (objects[x][y] >= 'A' && objects[x][y] <= 'G') {
    if (objects[x][y] == 'A') {
#if SOUND
      if (sound) {
        SoundPlay(3, (channel++)%NUMCHANNELS);
      }
#endif
      theBoni->removeBonus(x, y);
      if (theBoni->getNumAs() < 10) {
        theBoni->addBonus('A', 0);
        dscore += theBoni->getNumAs() * level;
        myBar->update(lives, score, 2);
        growing += (int)pow(1.6, theBoni->getNumAs() + 1) + 1;
        theBoni->drawAll();
      } else {
        dscore += theBoni ->getNumAs() * level;
        myBar->update(lives, score, 2);
      }
    }
    if (objects[x][y] == 'B') {
#if SOUND
      if (sound) {  
        SoundPlay(4, (channel++)%NUMCHANNELS);
      }
#endif
      do {
        xx = rand() % (BWIDTH - 2) + 1;
        yy = rand() % (BHEIGHT - 2) + 1;
      } while (objects[xx][yy] != 'a');
      head->setX(xx);
      head->setY(yy);
    }
    if (objects[x][y] == 'C') {
#if SOUND
      if (sound) {
        SoundPlay(4, (channel++)%NUMCHANNELS);
      }
#endif
      head->setX(warpx);
      head->setY(warpy);
    }
    if (objects[x][y] == 'E') {
#if SOUND
      if (sound) {
        SoundPlay(6, (channel++)%NUMCHANNELS);
      }
#endif
      theBoni->removeBonus(x, y);
      cutInHalf();
      myBar->update(lives, score, 2);
    }
    if (objects[x][y] == 'F') {
#if SOUND
      if (sound) {
        SoundPlay(7, (channel++)%NUMCHANNELS);
      }
#endif
      theBoni->removeBonus(x, y);
      doubleIt();
      myBar->update(lives, score, 2);
    }
    if (objects[x][y] == 'G') {
#if SOUND
      if (sound) {
        SoundPlay(8, (channel++)%NUMCHANNELS);
      }
#endif
      theBoni->removeBonus(x, y);
      if (lives < 9) {
        lives++;
        myBar->update(lives, score, 1);
      }
    }
  }
  return 0;
}

void worm::setGrow(int growing1) {
  growing += growing1;
}

void worm::setShrink(int shrinking1) {
  shrinking += shrinking1;
}

int worm::getLives() {
  return(lives);
}

int worm::getScore() {
  return(score + dscore);
}

void worm::setLives(int alives) {
  lives = alives;
}

void worm::cutInHalf() {
  int count = 1;

  count = (length == 1) ? 0 : length >> 1;
  length -= count;
  dscore += count * level + 5;
  myBar->update(lives, score, 2);
  shrinking += count;
}

void worm::doubleIt() {
  dscore += length * level;
  growing = length;
}

void worm::setRealScore(int fibbee) {
  score = fibbee;
}

void worm::setScore(int fibbee) {
  dscore = fibbee;
}

void worm::resetMelt() {
  meltCount = MELTSIZE;
  meltFlag = meltDelay;
}

int worm::unPause() {
  if (length) {
    if (!flashCount) {
      flashCount = FLASHCOUNT;
      return 1;
    }
    if (!(flashCount % 2) && !flashDelay) {
      head->erase();
      flashCount--;
      flashDelay = fps/8;
      return 0;
    }
    if (!flashDelay) {
      head->draw(&wormImage);
      flashCount--;
      flashDelay = fps/8;
      return 0;
    }
    flashDelay--;
    return 0;
  } else
    return 1;
}
